@@ -479,8 +479,8 @@ class Parameter(Enum):
 
     # VISIBILITY
     # ---- distance ----
-    VISIBILITY_INDICATOR = "VISIBILITY_INDICATOR"
-    VISIBILITY = "VISIBILITY"  # through clouds, fog, etc
+    VISIBILITY_RANGE_INDICATOR = "VISIBILITY_INDICATOR"
+    VISIBILITY_RANGE = "VISIBILITY"  # through clouds, fog, etc
     # ---- probability ----
     PROBABILITY_VISIBILITY_BELOW_1000_M = "vv10"
 
@@ -504,6 +504,7 @@ class Parameter(Enum):
     WEATHER_SIGNIFICANT_OPTIONAL_LAST_24H = "WEATHER_SIGNIFICANT_OPTIONAL_LAST_24H"
     # TODO: ECCC parameter, same as significant weather?
     WEATHER = "WEATHER"
+    WEATHER_TEXT = "WEATHER_TEXT"
 
     # WEATHER PHENOMENA
     # ---- averaged ----
@@ -560,6 +561,7 @@ class Parameter(Enum):
     WIND_GUST_MAX = "WIND_GUST_MAX"
     WIND_GUST_MAX_LAST_1H = "WIND_GUST_MAX_LAST_1H"
     WIND_GUST_MAX_LAST_3H = "WIND_GUST_MAX_LAST_3H"
+    WIND_GUST_MAX_LAST_6H = "WIND_GUST_MAX_LAST_6H"
     WIND_GUST_MAX_LAST_12H = "WIND_GUST_MAX_LAST_12H"
     WIND_SPEED_MIN = "WIND_SPEED_MIN"
     WIND_SPEED_ROLLING_MEAN_MAX = "WIND_SPEED_ROLLING_MEAN_MAX"
@@ -595,3 +597,21 @@ class Parameter(Enum):
     # ---- error ----
     ERROR_ABSOLUTE_WIND_SPEED = "ERROR_ABSOLUTE_WIND_SPEED"
     ERROR_ABSOLUTE_WIND_DIRECTION = "ERROR_ABSOLUTE_WIND_DIRECTION"
+
+    # Waterways related parameters
+    TEMPERATURE_WATER = "TEMPERATURE_WATER"
+    ELECTRIC_CONDUCTIVITY = "ELECTRIC_CONDUCTIVITY"
+    WATER_LEVEL = "WATER_LEVEL"
+    GROUNDWATER_LEVEL = "GROUNDWATER_LEVEL"
+    DISCHARGE = "DISCHARGE"
+    FLOW_SPEED = "FLOW_SPEED"
+    OXYGEN_LEVEL = "OXYGEN_LEVEL"
+    TURBIDITY = "TURBIDITY"
+    PRECIPITATION_INTENSITY = "PRECIPITATION_INTENSITY"
+    WAVE_PERIOD = "WAVE_PERIOD"
+    CLEARANCE_HEIGHT = "CLEARANCE_HEIGHT"
+    CURRENT = "CURRENT"
+    WAVE_HEIGHT_SIGN = "WAVE_HEIGHT_SIGN"
+    WAVE_HEIGHT_MAX = "WAVE_HEIGHT_MAX"
+    PH_VALUE = "PH_VALUE"
+    CHLORID_CONCENTRATION = "CHLORID_CONCENTRATION"
